package com.ruoyi.project.VRdemo.service.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.VRdemo.domain.Research;
import com.ruoyi.project.VRdemo.mapper.DataExcelMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.VRdemo.mapper.ResearchMapper;
import com.ruoyi.project.VRdemo.domain.Research;
import com.ruoyi.project.VRdemo.service.IResearchService;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

/**
 * 研究Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-06-23
 */
@Service
public class ResearchServiceImpl implements IResearchService 
{
    @Autowired
    private ResearchMapper researchMapper;

    @Autowired
    private DataExcelMapper dataExcelMapper;
    private static final Logger logger = LoggerFactory.getLogger(Research.class);

    private static final int BATCH_COUNT = 10000;

    /**
     * 查询研究
     * 
     * @param id 研究主键
     * @return 研究
     */
    @Override
    public Research selectResearchById(Long id)
    {
        return researchMapper.selectResearchById(id);
    }

    /**
     * 查询研究列表
     * 
     * @param research 研究
     * @return 研究
     */
    @Override
    public List<Research> selectResearchList(Research research)
    {
        return researchMapper.selectResearchList(research);
    }

    /**
     * 新增研究
     * 
     * @param research 研究
     * @return 结果
     */
    @Override
    public int insertResearch(Research research)
    {
        return researchMapper.insertResearch(research);
    }

    /**
     * 修改研究
     * 
     * @param research 研究
     * @return 结果
     */
    @Override
    public int updateResearch(Research research)
    {
        return researchMapper.updateResearch(research);
    }

    /**
     * 批量删除研究
     * 
     * @param ids 需要删除的研究主键
     * @return 结果
     */
    @Override
    public int deleteResearchByIds(Long[] ids)
    {
        return researchMapper.deleteResearchByIds(ids);
    }

    /**
     * 删除研究信息
     * 
     * @param id 研究主键
     * @return 结果
     */
    @Override
    public int deleteResearchById(Long id)
    {
        return researchMapper.deleteResearchById(id);
    }

    //EasyExcel批量导入
    @Override
    @Transactional(isolation = Isolation.READ_COMMITTED)
    public String importResearch(List<Research> researchList, Boolean updateSupport) {
        if (StringUtils.isNull(researchList) || researchList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }

        List<Research> batchDataList = new ArrayList<>();
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        // 遍历用户数据列表，逐条处理数据
        for (Research research : researchList) {
            try {
                // 对每一条数据进行处理
                // 根据title字段判断数据合法性
                if (StringUtils.isEmpty(research.getTitle())) {
                    throw new ServiceException("模板标题不能为空！");
                }

                // 如果更新支持，检查数据库中是否存在相同的记录，如果存在，则进行更新操作
                // 判断数据库中是否存在相同标题的记录
                Research existingResearch = null;
                if(updateSupport){
                    String titleAsString = research.getTitle();
                    existingResearch= dataExcelMapper.selectResearchByTitle(titleAsString);
                }
                if (existingResearch != null) {
                    research.setUpdateBy(SecurityUtils.getLoginUser().getUsername());
                    research.setUpdateTime(DateUtils.getNowDate());
                    // 更新操作
                    dataExcelMapper.updateResearchByImport(research);
                    successNum++;
                    successMsg.append("成功更新数据：").append(research.getTitle()).append("\n");
                } else {
                    // 检查是否已经存在于 batchDataList 中，避免重复插入
                    if (!batchDataList.contains(research)) {
//                        research.setCreateBy(SecurityUtils.getLoginUser().getUsername());
                        research.setCreateTime(DateUtils.getNowDate());
                        batchDataList.add(research);
                    }
                    // 判断是否达到BATCH_COUNT，或者已经是最后一个数据
                    if (batchDataList.size() >= BATCH_COUNT || researchList.indexOf(research) == researchList.size() - 1) {
                        System.out.println( "batchDataList size: " + batchDataList);
                        // 执行批量插入操作
                        dataExcelMapper.insertResearchDataList(batchDataList);
                        successNum += batchDataList.size();
                        for (Research insertedResearch : batchDataList) {
                            successMsg.append("成功导入数据：").append(insertedResearch.getTitle()).append("\n");
                        }
                        batchDataList.clear();
                    }
                }
            } catch (Exception e) {
                // 输出异常信息和上下文信息到日志
                logger.error("导入数据时发生异常：", e);
                logger.error("异常发生时的数据：{}", research); // 这里的 study 是导致异常的数据对象
                // 处理导入失败的情况
                failureNum++;
                failureMsg.append("导入失败，模板标题为：").append(research.getTitle()).append("\n");
            }
        }
        // 判断是否有导入失败的数据
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确或插入数据库出错，错误如下：\n");
            throw new ServiceException(failureMsg.toString());
        } else {
            // 若全部数据导入成功，则返回成功导入的信息
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：\n");
        }
        return successMsg.toString();
    }
}
