package com.ruoyi.project.VRdemo.service.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.VRdemo.mapper.DataExcelMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.VRdemo.mapper.ThesisMapper;
import com.ruoyi.project.VRdemo.domain.Thesis;
import com.ruoyi.project.VRdemo.service.IThesisService;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

/**
 * 论文Service业务层处理
 *
 * @author ruoyi
 * @date 2024-06-23
 */
@Service
public class ThesisServiceImpl implements IThesisService {
    @Autowired
    private ThesisMapper thesisMapper;

    @Autowired
    private DataExcelMapper dataExcelMapper;
    private static final Logger logger = LoggerFactory.getLogger(Thesis.class);

    private static final int BATCH_COUNT = 10000;

    /**
     * 查询论文
     *
     * @param id 论文主键
     * @return 论文
     */
    @Override
    public Thesis selectThesisById(Long id) {
        return thesisMapper.selectThesisById(id);
    }

    /**
     * 查询论文列表
     *
     * @param thesis 论文
     * @return 论文
     */
    @Override
    public List<Thesis> selectThesisList(Thesis thesis) {
        return thesisMapper.selectThesisList(thesis);
    }

    /**
     * 新增论文
     *
     * @param thesis 论文
     * @return 结果
     */
    @Override
    public int insertThesis(Thesis thesis) {
        return thesisMapper.insertThesis(thesis);
    }

    /**
     * 修改论文
     *
     * @param thesis 论文
     * @return 结果
     */
    @Override
    public int updateThesis(Thesis thesis) {
        return thesisMapper.updateThesis(thesis);
    }

    /**
     * 批量删除论文
     *
     * @param ids 需要删除的论文主键
     * @return 结果
     */
    @Override
    public int deleteThesisByIds(Long[] ids) {
        return thesisMapper.deleteThesisByIds(ids);
    }

    /**
     * 删除论文信息
     *
     * @param id 论文主键
     * @return 结果
     */
    @Override
    public int deleteThesisById(Long id) {
        return thesisMapper.deleteThesisById(id);
    }

    //EasyExcel批量导入
    @Override
    @Transactional(isolation = Isolation.READ_COMMITTED)
    public String importThesis(List<Thesis> thesisList, Boolean updateSupport) {
        if (StringUtils.isNull(thesisList) || thesisList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }

        List<Thesis> batchDataList = new ArrayList<>();
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();

        // 遍历用户数据列表，逐条处理数据
        for (Thesis thesis : thesisList) {
            try {
                // 对每一条数据进行处理
                // 根据title字段判断数据合法性
                if (StringUtils.isEmpty(thesis.getTitle())) {
                    throw new ServiceException("模板标题不能为空！");
                }

                // 如果更新支持，检查数据库中是否存在相同的记录，如果存在，则进行更新操作
                // 判断数据库中是否存在相同标题的记录
                Thesis existingThesis = null;
                if (updateSupport) {
                    String titleAsString = thesis.getTitle();
                    existingThesis = dataExcelMapper.selectThesisByTitle(titleAsString);
                }
                if (existingThesis != null) {
                    thesis.setUpdateBy(SecurityUtils.getLoginUser().getUsername());
                    thesis.setUpdateTime(DateUtils.getNowDate());
                    // 更新操作
                    dataExcelMapper.updateThesisByImport(thesis);
                    successNum++;
                    successMsg.append("成功更新数据：").append(thesis.getTitle()).append("\n");
                } else {
                    // 检查是否已经存在于 batchDataList 中，避免重复插入
                    if (!batchDataList.contains(thesis)) {
//                        thesis.setCreateBy(SecurityUtils.getLoginUser().getUsername());
                        thesis.setCreateTime(DateUtils.getNowDate());
                        batchDataList.add(thesis);
                    }
                    // 判断是否达到BATCH_COUNT，或者已经是最后一个数据
                    if (batchDataList.size() >= BATCH_COUNT || thesisList.indexOf(thesis) == thesisList.size() - 1) {
                        System.out.println("batchDataList size: " + batchDataList);
                        // 执行批量插入操作
                        dataExcelMapper.insertThesisDataList(batchDataList);
                        successNum += batchDataList.size();
                        for (Thesis insertedThesis : batchDataList) {
                            successMsg.append("成功导入数据：").append(insertedThesis.getTitle()).append("\n");
                        }
                        batchDataList.clear();
                    }
                }
            } catch (Exception e) {
                // 输出异常信息和上下文信息到日志
                logger.error("导入数据时发生异常：", e);
                logger.error("异常发生时的数据：{}", thesis); // 这里的 study 是导致异常的数据对象
                // 处理导入失败的情况
                failureNum++;
                failureMsg.append("导入失败，模板标题为：").append(thesis.getTitle()).append("\n");
            }
        }
        // 判断是否有导入失败的数据
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确或插入数据库出错，错误如下：\n");
            throw new ServiceException(failureMsg.toString());
        } else {
            // 若全部数据导入成功，则返回成功导入的信息
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：\n");
        }
        return successMsg.toString();
    }
}
